import com.imptt.proptt.common.*;
import com.imptt.proptt.common.plugins.V1.*;
import java.util.Arrays;
import java.io.*;

public class CryptoPluginSample implements ICryptoPlugin
{
	public int getInterfaceVersion()
	{
		println("CryptoPluginSample::getInterfaceVersion");
		return INTERFACE_VERSION;
	}
	
	public int getType()
	{
		println("CryptoPluginSample::getType");
		return TYPE_CRYPTO_PLUGIN;
	}

	public boolean initialize(String arg)
	{
		println("CryptoPluginSample::initialize arg="+arg);
		return true;
	}
	
	//methods
	public boolean canSupportPTT()
	{
		println("CryptoPluginSample::canSupportPTT");
		return true;
	}

	public boolean canSupportChat()
	{
		println("CryptoPluginSample::canSupportChat");
		return true;
	}

	public boolean canSupportLocation()
	{
		println("CryptoPluginSample::canSupportLocation");
		return true;
	}

	public boolean canSupportVideoShare()
	{
		println("CryptoPluginSample::canSupportVideoShare");
		return true;
	}

	//For PTT
	public void beginPTT(int pttChannelId, int lockIndex)
	{
		println("CryptoPluginSample::beginPTT pttChannelId="+pttChannelId+", lockIndex="+lockIndex);
	}

	public byte[] encryptPTT(int pttChannelId, int lockIndex, byte[] inputData, int inputDataOffset, int inputDataLength)
	{
		println("CryptoPluginSample::encryptPTT pttChannelId="+pttChannelId+", lockIndex="+lockIndex);

		int resultSize = ((inputDataLength + 15)/16)*16;
		byte[] result = new byte[resultSize];
        Arrays.fill(result, (byte) 0x00);
        System.arraycopy(inputData, inputDataOffset, result, 0, inputDataLength);
        byte[] temp = new byte[7];
        System.arraycopy(result, resultSize - 1 - 7, temp, 0, 7);
        System.arraycopy(result, 0, result, resultSize - 1 - 7, 7);
        System.arraycopy(temp, 0, result, 0, 7);
		return result;
	}

	public byte[] decryptPTT(int pttChannelId, int lockIndex, byte[] inputData, int inputDataOffset, int inputDataLength)
	{
		println("CryptoPluginSample::decryptPTT pttChannelId="+pttChannelId+", lockIndex="+lockIndex);

		byte[] result = new byte[inputDataLength];
        System.arraycopy(inputData, inputDataOffset, result, 0, inputDataLength);
        System.arraycopy(inputData, inputDataOffset, result, inputDataLength - 1 - 7, 7);
        System.arraycopy(inputData, inputDataOffset + inputDataLength - 1 - 7, result, 0, 7);
		return result;
	}

	public void endPTT(int pttChannelId, int lockIndex)
	{
		println("CryptoPluginSample::endPTT pttChannelId="+pttChannelId+", lockIndex="+lockIndex);
	}

	
	//For VideoShare
	public void beginVideoShare(int videoChannelId, String userId)
	{
		println("CryptoPluginSample::beginVideoShare videoChannelId="+videoChannelId+", userId="+userId);
	}

	public byte[] encryptVideoShare(int videoChannelId, String userId, byte[] inputData, int inputDataOffset, int inputDataLength)
	{
		println("CryptoPluginSample::encryptVideoShare videoChannelId="+videoChannelId+", userId="+userId);
		int resultSize = ((inputDataLength + 15)/16)*16;
		byte[] result = new byte[resultSize];
        Arrays.fill(result, (byte) 0x00);
        System.arraycopy(inputData, inputDataOffset, result, 0, inputDataLength);
        byte[] temp = new byte[7];
        System.arraycopy(result, resultSize - 1 - 7, temp, 0, 7);
        System.arraycopy(result, 0, result, resultSize - 1 - 7, 7);
        System.arraycopy(temp, 0, result, 0, 7);
		return result;
	}

	public byte[] decryptVideoShare(int videoChannelId, String userId, byte[] inputData, int inputDataOffset, int inputDataLength)
	{
		println("CryptoPluginSample::decryptVideoShare videoChannelId="+videoChannelId+", userId="+userId);
		byte[] result = new byte[inputDataLength];
        System.arraycopy(inputData, inputDataOffset, result, 0, inputDataLength);
        System.arraycopy(inputData, inputDataOffset, result, inputDataLength - 1 - 7, 7);
        System.arraycopy(inputData, inputDataOffset + inputDataLength - 1 - 7, result, 0, 7);
		return result;
	}

	public void endVideoShare(int videoChannelId, String userId)
	{
		println("CryptoPluginSample::endVideoShare videoChannelId="+videoChannelId+", userId="+userId);
	}

		
	//For Others
	public byte[] encryptOther(CommonConst.CryptoType type, byte[] inputData, int inputDataOffset, int inputDataLength)
	{
		println("CryptoPluginSample::encryptOther type="+type);
		int resultSize = ((inputDataLength + 15)/16)*16;
		byte[] result = new byte[resultSize];
        Arrays.fill(result, (byte) 0x00);
        System.arraycopy(inputData, inputDataOffset, result, 0, inputDataLength);
        byte[] temp = new byte[7];
        System.arraycopy(result, resultSize - 1 - 7, temp, 0, 7);
        System.arraycopy(result, 0, result, resultSize - 1 - 7, 7);
        System.arraycopy(temp, 0, result, 0, 7);
		return result;
	}

	public byte[] decryptOther(CommonConst.CryptoType type, byte[] inputData, int inputDataOffset, int inputDataLength)
	{
		println("CryptoPluginSample::decryptOther type="+type);
		byte[] result = new byte[inputDataLength];
        System.arraycopy(inputData, inputDataOffset, result, 0, inputDataLength);
        System.arraycopy(inputData, inputDataOffset, result, inputDataLength - 1 - 7, 7);
        System.arraycopy(inputData, inputDataOffset + inputDataLength - 1 - 7, result, 0, 7);
		return result;
	}

	void println(String data)
	{
		print(data+"\n");
	}

	void print(String data)
	{
		File file = new File("CryptoPluginSample.log");
		try {
			FileWriter fileWriter = new FileWriter(file, true);
			PrintWriter printWriter = new PrintWriter(fileWriter);
			printWriter.print(data);
			printWriter.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void dump(byte[] buff, int offset, int data_len)
    {
          
      int a;
      StringBuilder sb = new StringBuilder();
      println("DUMP_START : ===========================================");
      for(int i=offset; i<(data_len+offset); i++)
      {
        a = ((buff[i]>>4) & 0x0F);
        sb.append(printHalfByte(a));
        a = (buff[i] & 0x0F);
        sb.append(printHalfByte(a));
        sb.append(" ");
       if((i+1-offset)%16 == 0) {
    	   println(sb.toString());
    	   sb = new StringBuilder();
       }
      }
      if(sb.length()>0) println(sb.toString());
      println("DUMP_END : =============================================");
    }
	
	private String printHalfByte(int a)
    {
      if(a==0) return "0";
      else if(a==1) return "1";
      else if(a==2) return "2";
      else if(a==3) return "3";
      else if(a==4) return "4";
      else if(a==5) return "5";
      else if(a==6) return "6";
      else if(a==7) return "7";
      else if(a==8) return "8";
      else if(a==9) return "9";
      else if(a==10) return "A";
      else if(a==11) return "B";
      else if(a==12) return "C";
      else if(a==13) return "D";
      else if(a==14) return "E";
      else if(a==15) return "F";
      return "";
    }
}
